@@ -1,12 +1,13 @@
 #![allow(clippy::bool_comparison)]
 extern crate cargo_fuzzcheck;
-use cargo_fuzzcheck::*;
-use fuzzcheck_common::arg::*;
 use std::error::Error;
 use std::path::PathBuf;
 use std::process;
 use std::string::String;
 
+use cargo_fuzzcheck::*;
+use fuzzcheck_common::arg::*;
+
 const CARGO_ARGS_FLAG: &str = "cargo-args";
 
 fn main() -> Result<(), Box<dyn Error>> {
@@ -15,18 +16,14 @@ fn main() -> Result<(), Box<dyn Error>> {
     parser.optflag("", "lib", "Test only this package's library unit tests (default)");
     parser.optopt("", "bin", "Test only the specified binary", "<NAME>");
     parser.optopt("", "test", "Test only the specified test target", "<NAME>");
-
-    parser.optflag(
-        "",
-        "instrument-only-root",
-        "Instrument only the current package, not any of the dependencies",
-    );
+    parser.optflag("", "address-sanitizer", "Use AddressSanitizer");
     parser.optopt(
         "",
-        "exec-name",
-        "The name of the test executable that will be created by cargo. Necessary when using --instrument-only-root",
-        "<NAME>",
+        "profile",
+        "The profile to use to compile the fuzz test (default: release)",
+        "",
     );
+    parser.optflag("", "no-instrument-coverage", "Turn off coverage instrumentation");
 
     parser.opt(
         "",
@@ -63,6 +60,9 @@ fn main() -> Result<(), Box<dyn Error>> {
             return Err(Box::new(e));
         }
     };
+    let address_sanitizer = matches.opt_present("address-sanitizer");
+    let profile = matches.opt_str("profile").unwrap_or_else(|| "release".to_owned());
+    let no_instrument_coverage = matches.opt_present("no-instrument-coverage");
 
     // this won't crash because we `Arguments::from_matches` would have returned an error otherwise
     let target_name = &matches.free[0];
@@ -98,14 +98,6 @@ fn main() -> Result<(), Box<dyn Error>> {
         unreachable!();
     };
 
-    let instrument_only_root = matches.opt_present("instrument-only-root");
-    let name_exec = matches.opt_get::<String>("exec-name").unwrap();
-    if instrument_only_root && name_exec.is_none() {
-        return Err(Box::new(ArgumentsError::Validation(
-            "--exec-name must be provided when using --instrument-only-root".to_string(),
-        )));
-    }
-
     match args.command {
         FuzzerCommand::Fuzz => {
             if args.corpus_in.is_none() && matches.opt_present(NO_IN_CORPUS_FLAG) == false {
@@ -123,9 +115,11 @@ fn main() -> Result<(), Box<dyn Error>> {
             let exec = launch_executable(
                 target_name,
                 &args,
-                if instrument_only_root { name_exec } else { None },
                 &compiled_target,
                 &cargo_args,
+                address_sanitizer,
+                &profile,
+                !no_instrument_coverage,
                 &process::Stdio::inherit,
             )?;
             exec.wait_with_output()?;
@@ -136,6 +130,9 @@ fn main() -> Result<(), Box<dyn Error>> {
                 &args,
                 &compiled_target,
                 &cargo_args,
+                address_sanitizer,
+                &profile,
+                !no_instrument_coverage,
                 &process::Stdio::inherit,
             )?;
         }
@@ -143,9 +140,11 @@ fn main() -> Result<(), Box<dyn Error>> {
             let exec = launch_executable(
                 target_name,
                 &args,
-                if instrument_only_root { name_exec } else { None },
                 &compiled_target,
                 &cargo_args,
+                address_sanitizer,
+                &profile,
+                !no_instrument_coverage,
                 &process::Stdio::inherit,
             )?;
             exec.wait_with_output()?;
